#!/usr/bin/env bash
# vim:set ts=3:

NAME="$1"

# Bot Directory
BD=$HOME/.local/opt

if [[ -z $NAME ]]; then
	echo "usage: ${0##*/} <container-name>"
	exit 1
fi

if ! [[ $NAME =~ ^[[:alnum:]]*$ ]]; then
	echo "error: the container name must be alphanumeric"
	exit 1
fi

if [[ -e $BD/$NAME ]]; then
	echo "error: $BD/$NAME exists, refusing to continue"
	exit 1
fi

unset -v ID
ID=$(buildah containers --format "{{.ContainerID}}" --filter name="$NAME")

shopt -s -o errexit
shopt -s lastpipe

if [[ -z $ID ]]; then
	ID=$(buildah from --name "$NAME" scratch)
fi

# Release Version
RV=$(grep -o '[0-9]\+' /etc/redhat-release)

# Mount Point
MP=$(buildah mount $ID)

# Purposely leave the build directory to avoid having to re-download
# packages on retry.
function cleanup {
	buildah unmount $ID;
	echo
	echo "Failure. Container image '$NAME' failed to build."
	echo "The build directory for '$NAME' has not been removed."
	echo
}
trap cleanup ERR TERM INT HUP

# Install Command
IC="dnf install \
--assumeyes \
--releasever=$RV \
--installroot=$MP \
--setopt=install_weak_deps=false \
--nodocs \
--repo=fedora \
--repo=updates \
"

# Net::Async::Matrix dependencies
$IC \
perl-Config-General \
perl-Test \
perl-Test-Identity \
perl-Data-Dump \
perl-File-Slurper \
perl-IO-Async \
perl-IO-Socket-SSL \
perl-Digest-HMAC \
perl-HTML-Tree \
perl-HTML-Formatter \
perl-JSON-MaybeXS \
perl-Convert-Color \
perl-Color-Library \
perl-Font-AFM \

# JSON::API dependencies
$IC \
perl-JSON \
perl-JSON-XS \
perl-Test-Pod-Coverage \
perl-URI-Encode \
perl-Test-Fake-HTTPD \
perl-Module-Build-Tiny \
perl-IO-Capture \
perl-LWP-Protocol-https \

# CPAN
$IC \
perl-CPAN \
perl-CPAN-DistnameInfo \

# Run Command
RC="buildah run $ID --"

# Clear previous CPAN state if present
$RC rm -rf /root/.cpan

# Install a few Perl modules that are only available from CPAN
$RC env PERL_MM_USE_DEFAULT="1" cpan IO::Async::SSL Net::Async::Matrix
$RC env PERL_MM_USE_DEFAULT="1" cpan String::Tagged::Terminal
$RC env PERL_MM_USE_DEFAULT="1" cpan JSON::API && :
if [[ $? -ne 0 ]]; then
	echo
	echo "The JSON::API build failed, attempting upstream patch ..."
	echo

	PTH='/blib/lib/JSON/API.pm'
	MOD=$($RC find /root/.cpan -path "*$PTH" -print -quit); [[ -n $MOD ]]

	$IC patch

	# https://github.com/geofffranks/json-api/pull/6/commits/a96c84972b73bd1b13c0be48ea761a1c043ef3bc
	$RC patch -p 1 $MOD <<- 'END'
		diff -ur a/API.pm b/API.pm
		--- a/API.pm	2021-06-30 19:21:34.176727041 -0500
		+++ b/API.pm	2021-06-30 19:21:45.532833736 -0500
		@@ -17,6 +17,14 @@
		 	%EXPORT_TAGS = ();
		 }
		 
		+# not an object metohd
		+# build JSON object for encode/decode, disallow nonref
		+# to match previous non object usage, APIs should be returning
		+# JSON arrays or JSON objects
		+sub _build_json {
		+	JSON->new->allow_nonref(0);
		+}
		+
		 sub _debug
		 {
		 	my ($self, @lines) = @_;
		@@ -92,7 +100,8 @@
		 
		 	my $json = undef;
		 	eval {
		-		$json = to_json($obj);
		+		$self->{_json} ||= _build_json();
		+		$json = $self->{_json}->encode($obj);
		 		$self->_debug("JSON created: $json");
		 	} or do {
		 		if ($@) {
		@@ -114,7 +123,8 @@
		 	eval {
		 		$json = $self->{predecodehook}->($json)
		 			 if defined($self->{predecodehook});
		-		$obj = from_json($json);
		+		$self->{_json} ||= _build_json();
		+		$obj = $self->{_json}->decode($json);
		 		$self->_debug("Deserializing successful:",Dumper($obj));
		 	} or do {
		 		if ($@) {
	END

	$RC sh -c "cd ${MOD%$PTH} && ./Build test && ./Build install"
fi

# Cleanup
$RC rm -rf /var/cache/dnf
$RC rm -rf /root/.cpan
buildah unmount $ID;

# Configure the entrypoint and commit the image
buildah config --workingdir='/bot' $ID
buildah config --cmd './run' $ID
buildah commit --squash $ID $NAME | read IMAGE_ID
buildah rm $ID

echo
echo "Success! Container image '$NAME' has been created."
echo "The build directory for '$NAME' has been removed."
echo

buildah images --filter id=$IMAGE_ID
echo

# Install the bot.
mkdir -p $BD
cp -vr bot $BD/$NAME

# Define a service file for starting and stoping the bot.
mkdir -p $HOME/.config/systemd/user
cat << END > $HOME/.config/systemd/user/$NAME.service
[Unit]
Description=$NAME
Wants=network.target
After=network-online.target
AssertFileIsExecutable=$BD/$NAME/run
AssertFileNotEmpty=$BD/$NAME/config
StartLimitIntervalSec=6
StartLimitBurst=2

[Service]
Restart=always
RestartSec=1
ExecStart=/usr/bin/podman container run \\
	--mount=type=bind,src=$BD/$NAME,dst=/bot \\
	--replace --name=$NAME localhost/$NAME
ExecStop=/usr/bin/podman container stop \\
	--ignore --time 10 \\
	$NAME
ExecStopPost=/usr/bin/podman container rm \\
	--ignore --force \\
	$NAME
Type=simple

[Install]
WantedBy=default.target
END

echo "You will need to create $BD/$NAME/config before running '$NAME'."
echo "Edit $BD/$NAME/bot_commands.pm to add/remove commands that '$NAME' will respond to."
echo

echo "After it is configured, you can enter the following command to start '$NAME'."
echo "systemctl --user start $NAME"
echo

echo "Enter the following command to stop '$NAME'."
echo "systemctl --user stop $NAME"
echo
